@@ -1,4 +1,4 @@
 from .configlog import config_log
 from .version import __version__, commit_message
-from .client import SruClient, SruRequest, SruRecord
+from .client import SruClient, SruRequest, SruRecord, IzSruRecord
 from .briefrecord import BriefRec, BriefRecFactory
